/*
 * wath_kanji.c - WATH ѴѴ⥸塼
 *   Thu, Nov 21 1996 by Kouichi ABE (WALL)
 */

/*
 * Х():
 *	Ⱦѥʤν "" + "" = "" Ѵʤ
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "jiscodemap.h"
#include "sjiscodemap.h"
#include "euccodemap.h"
#include "kanacodemap.h"

/*
 * ޥ
 */
#define	BUF_SIZE	1024	/* ѥХåե	*/
#define	CHECK_SIZE	1024*16	/* ɿ¬ɤߥХåե */

#define	UNKNOWN		0	/* ɿ¬ǽ	*/
#define	ASCII		1	/* ASCII 		*/
#define	JIS		2	/* JIS 		*/
#define	SJIS		3	/* Shift-JIS 	*/
#define	EUC		4	/* ܸ EUC 	*/
#define	SJISorEUC	5	/* SJIS  EUC 餷	*/
#define	KANA		6	/* ¯˸Ⱦѥ	*/

#define	SS_POINT	4	/* ܸ EUC(SS)ɾ	*/
#define	EUC_POINT	3	/* ܸ EUC ɾ	*/
#define	SJIS_POINT	3	/* SJIS ɾ		*/
#define	KANA8_POINT	1	/* JIS8(Ⱦѥ)ɾ	*/

#define	ESC		'\033'	/* ʸ	*/
#define	SO		'\016'	/* Shift Out		*/
#define	SI		'\017'	/* Shift In		*/
#define	SS2		'\216'	/* ܸ EUC ɥåȣ	*/
#define	SS3		'\217'	/* ܸ EUC ɥåȣ	*/

#define	EOL		'\0'	/* ʸü	*/
#define	CR		'\r'	/* 	*/
#define	LF		'\n'	/* 	*/

#define	IsASCII(c)	(((c) >= 0x01 && (c) <= 0x7f) ? 1 : 0)
#define	IsJIS(c)	(((c) >= 0x21 && (c) <= 0x7e) ? 1 : 0)
#define	IsSJIS1(c)	((((c) >= 0x81 && (c) <= 0x9f) || \
			  ((c) >= 0xe0 && (c) <= 0xef)) ? 1 : 0)
#define	IsSJIS2(c)	((((c) >= 0x40 && (c) <= 0x7e) || \
			  ((c) >= 0x80 && (c) <= 0xfc)) ? 1 : 0)
#define	IsEUC(c)	(((c) >= 0xa1 && (c) <= 0xfe) ? 1 : 0)
#define	IsKANA7(c)	(((c) >= 0x21 && (c) <= 0x5f) ? 1 : 0)
#define	IsKANA8(c)	(((c) >= 0xa1 && (c) <= 0xdf) ? 1 : 0)

typedef	unsigned char	KChar;
typedef	unsigned char*	Kanji;

typedef struct {		/* ɤ߼ѹ¤ */
  char  buf[CHECK_SIZE];	/* ɤߥХåե       */
  int   index;			/* Хåեؼ       */
  int   size;			/* ɤ߹     */
  FILE  *fp;			/* ե롦ݥ   */
} Check;

struct 	JISEscapeSequence {		/* JIS ס	*/
	char	*escSeq;		/* ס	*/
	int	tabSkip;		/* åסХ		*/
	int	code;			/* ʸ			*/
} EscapeSeq[] = {
	{"\033$@",	3,	JIS},	/* JIS X 0208-1978 (JIS )	*/
	{"\033$(@",	4,	JIS},	/* JIS X 0208-1978 ISO		*/
	{"\033$B",	3,	JIS},	/* JIS X 0208-1983 (JIS )	*/
	{"\033$(B",	4,	JIS},	/* JIS X 0208-1983 ISO		*/
	{"\033&@\033$B",6,	JIS},	/* JIS X 0208-1990		*/
	{"\033$(D",	4,	JIS},	/* JIS X 0212-1990		*/
	{"\033(J",	3,	ASCII},	/* JIS X 0201-1976 roma		*/
	{"\033(B",	3,	ASCII},	/* ANSI X3.4 ascii		*/
	{"\033(I",	3,	KANA}	/* JIS X 0201-1976 katakana	*/
};
#define	JIS_ENTRY	(sizeof(EscapeSeq) / sizeof(struct JISEscapeSequence))

/*
 * ɽѿ
 */
static Check	chk;

/*
 * ؿ
 */
static Kanji	SJIS2JIS();
static Kanji	EUC2JIS();
static Kanji	JIS2EUC();
static Kanji	SJIS2EUC();
static Kanji	JIS2SJIS();
static Kanji	EUC2SJIS();
static int	GuessKanjiCode();
static int	PreRead();
static char	GetChar();
#ifdef	KANJI_DEBUG
static void	Usage();
#endif	/* KANJI_DEBUG */

/*
 * ؿ̾:	SJIS2JIS
 * ǽ:	SJIS ɤ JIS ɤѴ
 * :	SJIS ɤʸ
 * :	JIS ɤѴʸؤΥݥ
 */
static Kanji SJIS2JIS(str)
	Kanji	str;
{
  static KChar	jis[BUF_SIZE];
  KChar		c, c1, c2;
  register int	pos, state = ASCII;

  for (pos = 0; pos < BUF_SIZE; pos++) {	/* Хåեν	*/
    jis[pos] = EOL;			/* ʸüǥꥢ	*/
  }

  pos = 0;					/* ǥå	*/
  while ((c = *(str++)) != EOL) {
    if (IsASCII(c)) {				/* ASCII ɤ	*/
      if (state != ASCII) {
        jis[pos++] = ESC;			/* ASCII ɤѹ	*/
	jis[pos++] = '(';
	jis[pos++] = 'B';
	state	   = ASCII;			/*  ASCII 	*/
      }
      jis[pos++]   = c;				/* Τޤ޽		*/
    }
    else if (IsKANA8(c)) {			/* Ⱦѥ(JIS8)	*/
      if (state != JIS) {
        jis[pos++] = ESC;			/* JIS ɤѹ	*/
	jis[pos++] = '$';
	jis[pos++] = 'B';
	state	   = JIS;			/*  JIS 	*/
      }
      /* ȾѥʤѥʤѴ	*/
      jis[pos++] = code2jis_map[kana2code_map[c].b1];
      jis[pos++] = code2jis_map[kana2code_map[c].b2];
    }
    else if (IsSJIS1(c)) {			/* SJIS ɤ	*/
      if (state != JIS) {
        jis[pos++] = ESC;			/* JIS ɤѹ	*/
	jis[pos++] = '$';
	jis[pos++] = 'B';
	state	   = JIS;			/*  JIS 	*/
      }
      c1	= c;				/* SJIS 裱Х	*/
      c2	= *(str++);			/* SJIS 裲Х	*/
      if ((c2 >= 0x40 && c2 <= 0x7e) ||
	  (c2 >= 0x80 && c2 <= 0x9e)) {		/* JIS ɤѴ	*/
	jis[pos++] = code2jis_map[sjis2code_map1_l[c1]];
	jis[pos++] = code2jis_map[sjis2code_map2[c2]];
      }
      else if (c2 >= 0x9f && c2 <= 0xfc) {	/* JIS ɤѴ	*/
	jis[pos++] = code2jis_map[sjis2code_map1_r[c1]];
	jis[pos++] = code2jis_map[sjis2code_map2[c2]];
      }
      else {
        /* 褿饨顼	*/
      }
    }
  }
  return(jis);
}

/*
 * ؿ̾:	EUC2JIS
 * ǽ:	ܸ EUC  JIS ɤѴ
 * :	ܸ EUC ɤʸ
 * :	JIS ɤѴʸؤΥݥ
 */
static Kanji EUC2JIS(str)
	Kanji	str;
{
  static KChar	jis[BUF_SIZE];
  KChar		c;
  register int	pos, state = ASCII;

  for (pos = 0; pos < BUF_SIZE; pos++) {	/* Хåեν	*/
    jis[pos] = EOL;			/* ʸüǥꥢ	*/
  }

  pos = 0;					/* ǥå	*/
  while ((c = *(str++)) != EOL) {
    if (IsASCII(c)) {				/* ASCII ɤ	*/
      if (state != ASCII) {
        jis[pos++] = ESC;			/* ASCII ɤѹ	*/
	jis[pos++] = '(';
	jis[pos++] = 'B';
	state	   = ASCII;			/*  ASCII 	*/
      }
      jis[pos++]   = c;				/* Τޤ޽		*/
    }
    else if ((char)c == SS2) {			/* Ⱦѥ(JIS8)	*/
      if (state != JIS) {
        jis[pos++] = ESC;			/* JIS ɤѹ	*/
	jis[pos++] = '$';
	jis[pos++] = 'B';
	state	   = JIS;			/*  JIS 	*/
      }
      c = *(str++);
      /* ȾѥʤѥʤѴ	*/
      jis[pos++] = code2jis_map[kana2code_map[c].b1];
      jis[pos++] = code2jis_map[kana2code_map[c].b2];
    }
    else if (IsEUC(c)) {			/* ܸ EUC ɤ*/
      if (state != JIS) {
        jis[pos++] = ESC;			/* JIS ɤѹ	*/
	jis[pos++] = '$';
	jis[pos++] = 'B';
	state	   = JIS;			/*  JIS 	*/
      }
      jis[pos++]   = c & 0x7f;			/* 8bit ܤ򥯥ꥢ	*/
    }
  }
  return(jis);
}

/*
 * ؿ̾:	JIS2EUC
 * ǽ:	JIS ɤܸ EUC ɤѴ
 * :	JIS ɤʸ
 * :	ܸ EUC ɤѴʸؤΥݥ
 */
static Kanji JIS2EUC(str)
	Kanji	str;
{
  static KChar	euc[BUF_SIZE];
  KChar		c;
  register int	i, pos, state = ASCII;

  for (pos = 0; pos < BUF_SIZE; pos++) {	/* Хåեν	*/
    euc[pos] = EOL;			/* ʸüǥꥢ	*/
  }

  pos = 0;					/* ǥå	*/
  while ((c = *(str++)) != EOL) {
    if (c == ESC) {	/* JIS ɤΥס󥹤򥹥å	*/
      str--;					/* ݥ󥿤᤹	*/
      for (i = 0; i < JIS_ENTRY; i++) {
        if (strncmp(str, EscapeSeq[i].escSeq, (size_t)EscapeSeq[i].tabSkip) == 0) {
	  state = EscapeSeq[i].code;
	  str  += EscapeSeq[i].tabSkip;
	  break;
	}
      }
      continue;
    }
    switch (state) {
      case ASCII:				/* ASCII 		*/
	euc[pos++] = c;				/* Τޤ޽		*/
	break;
      case JIS:					/* JIS 		*/
	euc[pos++] = c | 0x80;			/* 8bit ܤΩƤ	*/
	break;
      case KANA:				/* Ⱦѥ		*/
	/* ȾѥʤѥʤѴ	*/
	euc[pos++] = code2euc_map[kana2code_map[c].b1];
	euc[pos++] = code2euc_map[kana2code_map[c].b2];
	break;
      default:					/* 褿饨顼	*/
	break;
    }
  }

  return(euc);
}

/*
 * ؿ̾:	SJIS2EUC
 * ǽ:	SJIS ɤܸ EUC ɤѴ
 * :	SJIS ɤʸ
 * :	ܸ EUC ɤѴʸؤΥݥ
 */
static Kanji SJIS2EUC(str)
	Kanji	str;
{
  static KChar	euc[BUF_SIZE];
  KChar		c, c1, c2;
  register int	pos;

  for (pos = 0; pos < BUF_SIZE; pos++) {	/* Хåեν	*/
    euc[pos] = EOL;			/* ʸüǥꥢ	*/
  }

  pos = 0;					/* ǥå	*/
  while ((c = *(str++)) != EOL) {
    if (IsASCII(c)) {				/* ASCII ɤ	*/
      euc[pos++] = c;				/* Τޤ޽		*/
    }
    else if (IsKANA8(c)) {			/* JIS8 Ⱦѥ	*/
      /* ȾѥʤѥʤѴ	*/
      euc[pos++] = code2euc_map[kana2code_map[c].b1];
      euc[pos++] = code2euc_map[kana2code_map[c].b2];
    }
    else if (IsSJIS1(c)) {			/* SJIS ɤ	*/
      c1 = c;					/* SJIS 裱Х	*/
      c2 = *(str++);				/* SJIS 裲Х	*/
      if ((c2 >= 0x40 && c2 <= 0x7e) ||
	  (c2 >= 0x80 && c2 <= 0x9e)) {	/* ܸ EUC ɤѴ	*/
	euc[pos++] = code2euc_map[sjis2code_map1_l[c1]];
	euc[pos++] = code2euc_map[sjis2code_map2[c2]];
      }
      else if (c2 >= 0x9f && c2 <= 0xfc) {	/* ܸ EUC ɤѴ */
	euc[pos++] = code2euc_map[sjis2code_map1_r[c1]];
	euc[pos++] = code2euc_map[sjis2code_map2[c2]];
      }
      else {
	/* 褿饨顼	*/
      }
    }
  }

  return(euc);
}

/*
 * ؿ̾:	JIS2SJIS
 * ǽ:	JIS ɤ SJIS ɤѴ
 * :	JIS ɤʸ
 * :	SJIS ɤѴʸؤΥݥ
 */
static Kanji JIS2SJIS(str)
	Kanji	str;
{
  static KChar	sjis[BUF_SIZE];
  KChar		c, c1, c2;
  register int	i, pos, state = ASCII;

  for (pos = 0; pos < BUF_SIZE; pos++) {	/* Хåեν	*/
    sjis[pos] = EOL;			/* ʸüǥꥢ	*/
  }

  pos = 0;					/* ǥå	*/
  while ((c = *(str++)) != EOL) {
    if (c == ESC) {	/* JIS ɤΥס󥹤򥹥å	*/
      str--;					/* ݥ󥿤᤹	*/
      for (i = 0; i < JIS_ENTRY; i++) {
        if (strncmp(str, EscapeSeq[i].escSeq, (size_t)EscapeSeq[i].tabSkip) == 0) {
	  state = EscapeSeq[i].code;
	  str  += EscapeSeq[i].tabSkip;
	  break;
	}
      }
      continue;
    }
    switch (state) {
      case ASCII:				/* ASCII 		*/
	sjis[pos++] = c;			/* Τޤ޽		*/
	break;
      case JIS:					/* JIS 		*/
	c1 = c;					/* JIS 裱Х	*/
	c2 = *(str++);				/* JIS 裲Х	*/
	if ((int)c1 % 2 == 0) {			/* 裱ХȤ϶	*/
	  sjis[pos++] = code2sjis_map1[jis2code_map[c1]];
	  sjis[pos++] = code2sjis_map2_e[jis2code_map[c2]];
	}
	else {					/* 裱ХȤϴ	*/
	  sjis[pos++] = code2sjis_map1[jis2code_map[c1]];
	  sjis[pos++] = code2sjis_map2_o[jis2code_map[c2]];
	}
	break;
      case KANA:				/* Ⱦѥ		*/
	/* ȾѥʤѥʤѴ	*/
	sjis[pos++] = code2sjis_map1[kana2code_map[c].b1];
	sjis[pos++] = code2sjis_map2_o[kana2code_map[c].b2];
	break;
      default:					/* 褿饨顼	*/
	break;
    }
  }

  return(sjis);
}

/*
 * ؿ̾:	EUC2SJIS
 * ǽ:	ܸ EUC ɤ SJIS ɤѴ
 * :	ܸ EUC ɤʸ
 * :	SJIS ɤѴʸؤΥݥ
 */
static Kanji EUC2SJIS(str)
	Kanji	str;
{
  static KChar	sjis[BUF_SIZE];
  KChar		c, c1, c2;
  register int	pos;

  for (pos = 0; pos < BUF_SIZE; pos++) {	/* Хåեν	*/
    sjis[pos] = EOL;			/* ʸüǥꥢ	*/
  }

  pos = 0;					/* ǥå	*/
  while ((c = *(str++)) != EOL) {
    if ((char)c == SS2) {			/* Ⱦѥʤ		*/
      c = *(str++);
      /* ȾѥʤѥʤѴ	*/
      sjis[pos++] = code2sjis_map1[kana2code_map[c].b1];
      sjis[pos++] = code2sjis_map2_o[kana2code_map[c].b2];
    }
    else if (IsASCII(c)) {			/* ASCII ɤ	*/
      sjis[pos++] = c;				/* Τޤ޽		*/
    }
    else if (IsEUC(c)) {			/* ܸ EUC ɤ*/
      c1 = c;					/* ܸ EUC 裱Х*/
      c2 = *(str++);				/* ܸ EUC 裲Х*/
      if ((int)c1 % 2 == 0) {			/* 裱ХȤ϶	*/
	sjis[pos++] = code2sjis_map1[euc2code_map[c1]];
	sjis[pos++] = code2sjis_map2_e[euc2code_map[c2]];
      }
      else {					/* 裱ХȤϴ	*/
	sjis[pos++] = code2sjis_map1[euc2code_map[c1]];
	sjis[pos++] = code2sjis_map2_o[euc2code_map[c2]];
      }
    }
  }

  return(sjis);
}

/*
 * ؿ̾:	GuessKanjiCode
 * ǽ:	ɤ¬
 * :	Ǥդʸ
 * :	¬
 */
static int GuessKanjiCode(str)
	Kanji	str;
{
  int	ascii_state, jis_state, euc_state, sjis_state;
  int	ascii_point, jis_point,	euc_point, sjis_point;
  KChar	c;

  ascii_state = jis_state = euc_state = sjis_state = 1;
  ascii_point = jis_point = euc_point = sjis_point = 0;

  while ((c = *(str++)) != EOL) {
    switch (ascii_state) {		/* ASCII Ѿ	*/
      case 1:
	if (IsASCII(c)) {				/* ASCII 	*/
	  ascii_state = 1;
	  ascii_point++;
	}
	else {						/* ¾	*/
	  ascii_state = 0;
	}
	break;
    }

    switch (jis_state) {			/* JIS Ѿ	*/
      case 1:
	if (c == ESC) {					/* ESC 	*/
	  jis_state = 2;
	}
	else if (IsASCII(c)) {				/* ASCII 	*/
	  jis_state = 1;
	}
	else {
	  jis_state = 0;				/* ¾	*/
	}
	break;
      case 2:
	if (c == '$') {			/* Хʸ	*/
	  jis_state = 3;
	}
	else if (c == '(') {		/* Хʸ	*/
	  jis_state = 4;
	}
	else {						/* ¾	*/
	  jis_state = 0;
	}
	break;
      case 3:
	if ((c == 'B') ||	/* JIS X 0208-1983 (JIS )	*/
	    (c == '@') ||	/* JIS X 0208-1978 (JIS )	*/
	    (c == 'D')) {	/* JIS X 0212-1990			*/
	  jis_state = 5;
	}
	else if (c == '(') {
	  jis_state = 3;				/* ISO 	*/
	}
	else {
	  jis_state = 0;				/* ¾	*/
	}
	break;
      case 4:
        if ((c == 'J') ||			/* JIS X 0208-1976 roma	*/
	    (c == 'B')) {			/* ANSI X 3.4 ascii	*/
	  jis_state = 1;
	}
	else if (c == 'I') {		/* JIS X 0201-1976 katakana	*/
	  jis_state = 6;
	}
	else {						/* ¾	*/
	  jis_state = 0;
	}
	break;
      case 5:
	if (IsJIS(c)) {				/* 2byte kanji code	*/
	  jis_state = 5;
	  jis_point++;
	}
	else if (c == ESC) {				/* ESC 	*/
	  jis_state = 2;
	}
	else {						/* ¾	*/
	  jis_state = 0;
	}
	break;
      case 6:
	if (IsKANA7(c) || 			/* JIS7 Ⱦѥ	*/
	    IsKANA8(c)) {			/* JIS8 Ⱦѥ	*/
	  jis_state = 6;
	  jis_point++;
	}
	else if (c == ESC) {
	  jis_state = 2;
	}
	else {
	  jis_state = 0;
	}
	break;
    }

    switch (euc_state) {
      case 1:
	if (IsASCII(c)) {				/* ASCII 	*/
	  euc_state = 1;
	}
	else if (IsEUC(c)) {			/* ܸ EUC 	*/
	  euc_state = 2;
	}
	else if ((char)c == SS2) {				/* SS2	*/
	  euc_point += SS_POINT;		/* ɾβû	*/
	  euc_state = 3;
	}
	else if ((char)c == SS3) {				/* SS3	*/
	  euc_point += SS_POINT;		/* ɾβû	*/
	  euc_state = 4;
	}
	else {						/* ¾	*/
	  euc_state = 0;
	}
	break;
      case 2:
	if (IsEUC(c)) {				/* ܸ EUC 	*/
	  euc_state = 1;
	  euc_point += EUC_POINT;
	}
	else {						/* ¾	*/
	  euc_state = 0;
	}
	break;
      case 3:
	if (IsKANA8(c)) {			/* JIS8 Ⱦѥ	*/
	  euc_state = 1;
	  euc_point += EUC_POINT;	/* ܸ EUC ݥȹ	*/
	}
	else {						/* ¾	*/
	  euc_state = 0;
	}
	break;
      case 4:
	if (IsEUC(c)) {				/* ܸ EUC 	*/
	  euc_state = 2;
	}
	else {						/* ¾	*/
	  euc_state = 0;
	}
	break;
    }

    switch (sjis_state) {
      case 1:
	if (IsASCII(c) || 				/* ASCII 	*/
	    IsKANA8(c)) {			/* JIS8 Ⱦѥ	*/
	  sjis_state = 1;
	  if (IsKANA8(c)) {		/* JIS8 Ⱦѥʤ⥫	*/
	    sjis_point += KANA8_POINT;
	  }
	}
	else if (IsSJIS1(c)) {		/* SJIS 裱Х	*/
	  sjis_state = 2;
	}
	else {						/* ¾	*/
	  sjis_state = 0;
	}
	break;
      case 2:
	if (IsSJIS2(c)) {		/* SJIS 裲Х	*/
	  sjis_state = 1;
	  sjis_point += SJIS_POINT;
	}
	else {
	  sjis_state = 0;				/* ¾	*/
	}
	break;
    }
  }

#ifdef	KANJI_DEBUG
  (void)fprintf(stderr, "\tASCII\tJIS\tSJIS\tEUC\n");
  (void)fprintf(stderr, "state:\t%d\t%d\t%d\t%d\n",
		ascii_state, jis_state, sjis_state, euc_state);
  (void)fprintf(stderr, "point:\t%d\t%d\t%d\t%d\n",
		ascii_point, jis_point, sjis_point, euc_point);
#endif	/* KANJI_DEBUG	*/
  if (jis_state && jis_point) {				/* JIS 	*/
    return(JIS);
  }

  if (euc_state && sjis_state) {	/* SJIS or ܸ EUC 	*/
    if (sjis_point > euc_point) {			/* SJIS 	*/
      return(SJIS);
    }
    else if (euc_point > sjis_point) {		/* ܸ EUC 	*/
      return(EUC);
    }
    else if ((sjis_point == euc_point) &&/* SJIS or ܸ EUC 	*/ 
	     (sjis_point) && (euc_point)) {
      return(SJISorEUC);
    }
  }

  if (sjis_state && sjis_point) {			/* SJIS 	*/
    return(SJIS);
  }

  if (euc_state && euc_point) {			/* ܸ EUC 	*/
    return(EUC);
  }

  if (ascii_state && ascii_point) {			/* ASCII 	*/
    return(ASCII);
  }

  return(UNKNOWN);	/* Ƚǽ(̾盧ʸ̵̣ʤϤ)	*/
}

/*
 * ؿ̾:	PreRead
 * ǽ:	ե뤫Хåեɤ߼
 * :	ե롦ݥ
 * :	ɤ߹
 */
int PreRead(fp)
	FILE	*fp;
{
  chk.size = fread(chk.buf, 1, CHECK_SIZE, fp);
  chk.index = 0;
  chk.fp = fp;

  return(chk.size);
}

/*
 * ؿ̾:	GetChar
 * ǽ:	ɤ߼Хåե or ե롦ݥ󥿤ʸɤ߽Ф
 * :	ʤ
 * :	饯ʸ or EOF
 */
char GetChar(void)
{
  register int	c;

  if (chk.index < chk.size) {
    return(chk.buf[chk.index++]);
  }
  else {
    if ((c = getc(chk.fp)) != EOF) {
      return((char)c);
    }
    else {
      return(EOF);
    }
  }
}

/*
 * ؿ̾:	JapaneseTextConvertTo
 * ǽ:	Ѵ
 * :	ɤ߹ߥե뵭һҡ񤭹ߥե뵭һҡ
 *		Ѵ̾ (JIS, SJIS, EUC)
 * :	ʤ
 */
void JapaneseTextConvertTo(rfd, wfd, kcode)
	int	rfd, wfd, kcode;
{
  register FILE	*fp;
  register int	i, n;
  char		c, buf[BUF_SIZE];
  Kanji		(*conv)() = NULL;

  fp = fdopen(rfd, "r"); /* ȥ꡼ե뵭һҤ˴Ϣդ	*/
  if (fp == NULL) {				/* Ϣդ		*/
    while ((n = read(rfd, buf, BUF_SIZE)) > 0) {/* Ѵʤ	*/
      (void)write(wfd, buf, strlen(buf));
    }
    return;
  }

  /* Ƚ̤Τɤ߹	*/
  n = PreRead(fp);

  /* Ƚ̤ȴѴؿ	*/
  switch (GuessKanjiCode(chk.buf)) {
    case JIS:					/* JIS 		*/
      if (kcode == SJIS) {			/* SJIS ɤѴ	*/
	(Kanji)conv = (Kanji)JIS2SJIS;
      }
      else if (kcode == EUC) {		/* ܸ EUC ɤѴ	*/
	(Kanji)conv = (Kanji)JIS2EUC;
      }
      break;
    case SJIS:					/* SJIS 		*/
      if (kcode == JIS) {			/* JIS ɤѴ	*/
	(Kanji)conv = (Kanji)SJIS2JIS;
      }
      else if (kcode == EUC) {		/* ܸ EUC ɤѴ	*/
	(Kanji)conv = (Kanji)SJIS2EUC;
      }
      break;
    case EUC:					/* ܸ EUC 	*/
      if (kcode == JIS) {			/* JIS ɤѴ	*/
	(Kanji)conv = (Kanji)EUC2JIS;
      }
      else if (kcode == SJIS) {			/* SJIS ɤѴ	*/
	(Kanji)conv = (Kanji)EUC2SJIS;
      }
      break;
    default:					/* Ѵʤ	*/
       break;
  }

  i = 0;					/* ǥå	*/
  while ((c = GetChar()) != EOF) {
    if (c != LF) {
      buf[i++] = c;
    }
    else {
      buf[i++] = LF;				/* ԥ˶ڤ	*/
      buf[i]   = EOL;				/* ʸüղ	*/
      if (conv != NULL) {			/* ѴԤ	*/
	(void)sprintf(buf, "%s", (Kanji)conv((Kanji)buf));
      }
      (void)write(wfd, buf, strlen(buf));	/* ե뵭һҤ˽	*/
      i = 0;
    }
  }
  (void)fclose(fp);			/* ե롦ݥ󥿤Ĥ	*/
}

/*
 * ؿ̾:	JapaneseStringConvertTo
 * ǽ:	Ѵ
 * :	ʸѴ̾ (JIS, SJIS, EUC)
 * :	ꤷɤѴʸؤΥݥ
 */
Kanji JapaneseStringConvertTo(str, kcode)
	char	*str;
	int	kcode;
{
  static KChar	buf[BUF_SIZE];
  Kanji		(*conv)() = NULL;

  /* Ƚ̤ȴѴؿ	*/
  switch (GuessKanjiCode(str)) {
    case JIS:					/* JIS 		*/
      if (kcode == SJIS) {			/* SJIS ɤѴ	*/
	(Kanji)conv = (Kanji)JIS2SJIS;
      }
      else if (kcode == EUC) {		/* ܸ EUC ɤѴ	*/
	(Kanji)conv = (Kanji)JIS2EUC;
      }
      break;
    case SJIS:					/* SJIS 		*/
      if (kcode == JIS) {			/* JIS ɤѴ	*/
	(Kanji)conv = (Kanji)SJIS2JIS;
      }
      else if (kcode == EUC) {		/* ܸ EUC ɤѴ	*/
	(Kanji)conv = (Kanji)SJIS2EUC;
      }
      break;
    case EUC:					/* ܸ EUC 	*/
      if (kcode == JIS) {			/* JIS ɤѴ	*/
	(Kanji)conv = (Kanji)EUC2JIS;
      }
      else if (kcode == SJIS) {			/* SJIS ɤѴ	*/
	(Kanji)conv = (Kanji)EUC2SJIS;
      }
      break;
    default:					/* Ѵʤ	*/
       break;
  }

  if (conv != NULL) {				/* ѴԤ	*/
    (void)sprintf(buf, "%s", (Kanji)conv((Kanji)str));
    return(buf);
  }
  else {				/* ѴԤʤ	*/
    return((Kanji)str);
  }
}

#ifdef	KANJI_DEBUG
/*
 * ؿ̾:	Usage
 * ǽ:	Ȥɽ
 * :	ץ̾
 * :	ʤ
 */
static void Usage(comm)
	char	*comm;
{
  (void)fprintf(stderr, "Usage: %s OPTION\n", comm);
  (void)fprintf(stderr, "  OPTION:\n");
  (void)fprintf(stderr, "    -j:	toJIS\n");
  (void)fprintf(stderr, "    -s:	toSJIS\n");
  (void)fprintf(stderr, "    -e:	toEUC\n\n");
  exit(0);
}

/*
 * ᥤؿ
 */
int main(argc, argv)
	int	argc;
	char	**argv;
{
  if (argc > 1) {
    if (strncmp(argv[1], "-j", 2) == 0) {
      JapaneseTextConvertTo(0, 1, JIS);
    }
    else if (strncmp(argv[1], "-s", 2) == 0) {
      JapaneseTextConvertTo(0, 1, SJIS);
    }
    else if (strncmp(argv[1], "-e", 2) == 0) {
      JapaneseTextConvertTo(0, 1, EUC);
    }
    else {
      Usage(argv[0]);
    }
  }
  else {
    Usage(argv[0]);
  }
}
#endif	/* KANJI_DEBUG */
